==> guikachu-0.3.tar.gz: guikachu-0.3/src/form-win.cc <==
//$Id: form-win.cc,v 1.6 2001/03/23 15:14:46 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "form-win.h"

#include "config.h"
#include <libgnome/libgnome.h>

#include "form-editor/label.h"

#include <gtk--/treeitem.h>
#include <gtk--/table.h>
#include <gtk--/button.h>
#include <gtk--/pixmap.h>

namespace Guikachu
{
    namespace GUI
    {
	namespace FormWindow_Helpers
	{
	    class WidgetTreeItem: public Gtk::TreeItem
	    {
		Resources::Widgets::Widget *res;
	    public:
		WidgetTreeItem (Resources::Widgets::Widget *res);
		SigC::Signal0<void> activate;
	    private:
		void update ();
		int button_press_cb (GdkEventButton *e);
	    };
	    
	    class TopTreeItem: public Gtk::TreeItem
	    {
		Resources::Form *res;
	    public:
		TopTreeItem (Resources::Form *res);
		SigC::Signal0<void> activate;
	    private:
		void update ();
		int button_press_cb (GdkEventButton *e);
	    };

	    WidgetTreeItem::WidgetTreeItem (Resources::Widgets::Widget *res_):
		Gtk::TreeItem (res_->get_id ()),
		res (res_)
	    {
		res->changed.connect (slot (this, &WidgetTreeItem::update));
		button_press_event.connect (slot (this, &WidgetTreeItem::button_press_cb));
	    }

	    void WidgetTreeItem::update ()
	    {
		remove ();
		add_label (res->get_id ());
	    }
	    
	    int WidgetTreeItem::button_press_cb (GdkEventButton *e)
	    {
		if (e->type == GDK_2BUTTON_PRESS && e->button == 1)
		    activate ();
		
		return false;
	    }
	    
	    TopTreeItem::TopTreeItem (Resources::Form *res_):
		Gtk::TreeItem (res_->get_id ()),
		res (res_)
	    {
		res->changed.connect (slot (this, &TopTreeItem::update));
		button_press_event.connect (slot (this, &TopTreeItem::button_press_cb));
	    }
	    
	    void TopTreeItem::update ()
	    {
		remove ();
		add_label (res->get_id ());
	    }
	    
	    int TopTreeItem::button_press_cb (GdkEventButton *e)
	    {
		if (e->type == GDK_2BUTTON_PRESS && e->button == 1)
		    activate ();
		    
		return false;
	    }
	}
    }
}

using namespace Guikachu::GUI;
using namespace Guikachu::Resources;

#include "pixmaps/label.xpm"

FormWindow::FormWindow (Form *res_):
    res (res_)
{
    gui = glade_xml_new (GLADEDIR "/guikachu.glade", 0);

    using namespace Gtk;
    
    window = wrap (GTK_WINDOW (glade_xml_get_widget (gui, "res_form")));
    window->delete_event.connect (slot (this, &FormWindow::delete_event_impl));

    property_window = new Gtk::Window (GTK_WINDOW_DIALOG);
    property_window->set_title (_("Properties"));
    property_window->delete_event.connect (slot (this, &FormWindow::property_delete_event_impl));
    property_window->set_policy (false, true, true);
    
    canvas = wrap (GNOME_CANVAS (glade_xml_get_widget (gui, "form_canvas")));
    Gtk::Tree *tree = wrap (GTK_TREE (glade_xml_get_widget (gui, "form_tree")));

    create_palette ();
    
    FormWindow_Helpers::TopTreeItem *item = new FormWindow_Helpers::TopTreeItem (res);
    item->show ();
    item->activate.connect (slot (this, &FormWindow::toplevel_activate_cb));
    tree->add (*manage (item));
    top_treeitem = item;
    
    form = new FormEditor::Form (res, this);
    property_window->add (*(form->get_editor ()));
    
    res->changed.connect (slot (this, &FormWindow::update));
    update ();
}

void FormWindow::create_palette ()
{
    using namespace Gtk;
    using namespace SigC;
    
    Table *palette = wrap (GTK_TABLE (glade_xml_get_widget (gui, "form_palette")));
    
#define PALETTE_ADD_BUTTON(type,tip,xpm,x,y)				\
    {									\
	Button *button = new Button;					\
	button->set_relief (GTK_RELIEF_NONE);				\
	button->add (*manage (new Gtk::Pixmap (xpm)));			\
	button->clicked.connect (					\
	    bind (slot (this, &FormWindow::add_widget_cb), type));	\
	tips.set_tip (*button, tip);					\
	palette->attach (*manage (button), x, (x + 1), y, (y + 1),	\
	    0, 0);							\
    }
    
    PALETTE_ADD_BUTTON(Widgets::WIDGET_LABEL,
		       _("Static text label"), label_xpm,
		       0, 0);
    
#undef PALETTE_ADD_BUTTON
}

FormWindow::~FormWindow ()
{
    window->destroy ();
    gtk_object_unref (GTK_OBJECT (gui));

    delete form;
    delete property_window;
}

void FormWindow::show ()
{
    window->show_all ();
    window->get_window ().raise ();

    property_window->show_all ();
    property_window->get_window ().raise ();
}

int FormWindow::delete_event_impl (GdkEventAny *e)
{
    window->hide ();
    property_window->hide ();
    return true;
}

int FormWindow::property_delete_event_impl (GdkEventAny *e)
{
    property_window->hide ();
    return true;
}

void FormWindow::update ()
{
    gchar *title_buf = g_strdup_printf (_("Form: %s"), res->get_id ().c_str ());
    window->set_title (title_buf);
    g_free (title_buf);

    Gtk::Tree *tree = top_treeitem->get_subtree ();
    if (tree)
    {
	tree->tree ().clear ();
    }
    
    const set<Widgets::Widget*>& widgets = res->get_widgets ();

    if (widgets.size ())
    {
	tree = new Gtk::Tree ();
	top_treeitem->set_subtree (*manage (tree));
	top_treeitem->expand ();
    }
    
    for (set<Widgets::Widget*>::const_iterator i = widgets.begin ();
	 i != widgets.end (); i++)
    {
	FormWindow_Helpers::WidgetTreeItem *item = new FormWindow_Helpers::WidgetTreeItem (*i);
	item->show ();
	item->activate.connect (bind (
	    slot (this, &FormWindow::widget_activate_cb),
	    *i));
	tree->add (*manage (item));
    }
}

void FormWindow::add_widget_cb (Widgets::Type type)
{
    res->create_widget (type);
}

void FormWindow::show_property_editor (Gtk::Widget *widget)
{
    if (property_window->get_child () != widget)
    {
	property_window->remove ();
	property_window->add (*widget);
    }
    
    property_window->show_all ();
    property_window->get_window ().raise ();
}

void FormWindow::toplevel_activate_cb ()
{
    show_property_editor (form->get_editor ());
}

void FormWindow::widget_activate_cb (Widgets::Widget *widget)
{
    show_property_editor (widget->get_editor ());
}

==> guikachu-0.4.tar.gz: guikachu-0.4/src/form-win.cc <==
//$Id: form-win.cc,v 1.12 2001/04/10 16:17:28 cactus Exp $ -*- c++ -*-

/* Guikachu Copyright (C) 2001 RDI Gerg <cactus@cactus.rulez.org>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "form-win.h"

#include "config.h"
#include <libgnome/libgnome.h>

#include <gtk--/treeitem.h>
#include <gtk--/table.h>
#include <gtk--/button.h>
#include <gtk--/pixmap.h>
#include <gtk--/menu.h>

#include "form-win-helpers.h"

using namespace Guikachu::GUI;
using namespace Guikachu::Resources;

#include "pixmaps/label.xpm"
#include "pixmaps/button.xpm"
#include "pixmaps/graffiti.xpm"

FormWindow::FormWindow (Form *res_):
    res (res_),
    edited_widget (0)
{
    gui = glade_xml_new (GLADEDIR "/guikachu.glade", 0);

    using namespace Gtk;
    
    window = wrap (GTK_WINDOW (glade_xml_get_widget (gui, "res_form")));
    window->delete_event.connect (slot (this, &FormWindow::delete_event_impl));

    property_window = new Gtk::Window (GTK_WINDOW_DIALOG);
    property_window->set_title (_("Widget properties"));
    property_window->delete_event.connect (slot (this, &FormWindow::property_delete_event_impl));
    property_window->set_policy (false, true, true);
    
    canvas = wrap (GNOME_CANVAS (glade_xml_get_widget (gui, "form_canvas")));
    Gtk::Tree *tree = wrap (GTK_TREE (glade_xml_get_widget (gui, "form_tree")));

    create_palette ();
    
    FormWindow_Helpers::TopTreeItem *item = new FormWindow_Helpers::TopTreeItem (res);
    item->show ();
    item->activate.connect (slot (this, &FormWindow::toplevel_activate_cb));
    tree->add (*manage (item));
    top_treeitem = item;
    
    form = new FormEditor::Form (res, this);
    form->show_editor.connect (slot (this, &FormWindow::show_form_property_editor));
    
    // Initial widget tree
    const set<Widgets::Widget*>& widgets = res->get_widgets ();
    for (set<Widgets::Widget*>::const_iterator i = widgets.begin ();
	 i != widgets.end (); i++)
	widget_created_cb (*i);

    
    res->changed.connect (slot (this, &FormWindow::update));
    res->widget_created.connect (slot (this, &FormWindow::widget_created_cb));
    res->widget_removed.connect (slot (this, &FormWindow::widget_removed_cb));
    update ();
}

void FormWindow::create_palette ()
{
    using namespace Gtk;
    using namespace SigC;
    
    Table *palette = wrap (GTK_TABLE (glade_xml_get_widget (gui, "form_palette")));
    
#define PALETTE_ADD_BUTTON(type,tip,xpm,x,y)				\
    {									\
	Button *button = new Button;					\
	button->set_relief (GTK_RELIEF_NONE);				\
	button->add (*manage (new Gtk::Pixmap (xpm)));			\
	button->clicked.connect (					\
	    bind (slot (this, &FormWindow::add_widget_cb), type));	\
	tips.set_tip (*button, tip);					\
	palette->attach (*manage (button), x, (x + 1), y, (y + 1),	\
	    0, 0);							\
    }
    
    PALETTE_ADD_BUTTON(Widgets::WIDGET_LABEL,
		       _("Static text label"), label_xpm,
		       0, 0);
    PALETTE_ADD_BUTTON(Widgets::WIDGET_BUTTON,
		       _("Clickable button"), button_xpm,
		       1, 0);
    PALETTE_ADD_BUTTON(Widgets::WIDGET_GRAFFITI,
		       _("Graffiti state indicator"), graffiti_xpm,
		       2, 0);
    
#undef PALETTE_ADD_BUTTON
}

FormWindow::~FormWindow ()
{
    window->destroy ();
    gtk_object_unref (GTK_OBJECT (gui));

    delete form;
    delete property_window;
}

void FormWindow::show ()
{
    window->show_all ();
    window->get_window ().raise ();

    if (edited_widget)
	edited_widget->show_editor ();
    else
	form->show_editor ();
}

int FormWindow::delete_event_impl (GdkEventAny *e)
{
    window->hide ();
    property_window->hide ();
    return true;
}

int FormWindow::property_delete_event_impl (GdkEventAny *e)
{
    property_window->hide ();
    return true;
}

void FormWindow::update ()
{
    gchar *title_buf = g_strdup_printf (_("Form: %s"), res->get_id ().c_str ());
    window->set_title (title_buf);
    g_free (title_buf);

}

void FormWindow::widget_created_cb (Widgets::Widget *widget)
{
    g_return_if_fail (widget != 0);

    using namespace SigC;
    
    widget->show_editor.connect (bind (
	slot (this, &FormWindow::show_property_editor),
	widget));
    
    // Add tree item
    Gtk::Tree *tree = top_treeitem->get_subtree ();
    if (!tree)
    {
	tree = new Gtk::Tree ();
	top_treeitem->set_subtree (*manage (tree));
	top_treeitem->expand ();
    }
    
    FormWindow_Helpers::WidgetTreeItem *item = new FormWindow_Helpers::WidgetTreeItem (widget);
    item->show ();
    item->activate.connect (bind (
	slot (this, &FormWindow::widget_activate_cb),
	widget));
    
    tree->add (*manage (item));
}

void FormWindow::widget_removed_cb (Widgets::Widget *widget)
{    
    if (widget == edited_widget)
	show_form_property_editor ();
    
    // Remove tree item
    Gtk::Tree::ItemList &itemlist = top_treeitem->get_subtree ()->tree ();
    Gtk::Tree::ItemList::iterator i = itemlist.begin ();
    
    while (i != itemlist.end () && (*i)->get_user_data () != widget)
	i++;

    g_return_if_fail (i != itemlist.end ());

    itemlist.erase (i);
}

void FormWindow::add_widget_cb (Widgets::Type type)
{
    res->create_widget (type);
}

void FormWindow::show_property_editor (Widgets::Widget *widget)
{
    if (edited_widget == widget)
	return;

    edited_widget = widget;
    Gtk::Widget *editor = widget->get_editor ();
    
    property_window->remove ();
    property_window->add (*editor);

    gchar *buffer = g_strdup_printf (_("Properties: %s"),
				     widget->get_id ().c_str ());
    property_window->set_title (buffer);
    g_free (buffer);
    
    property_window->show_all ();
    property_window->get_window ().raise ();
}

void FormWindow::show_form_property_editor ()
{
    edited_widget = 0;
    
    Gtk::Widget *editor = form->get_editor ();
    
    if (property_window->get_child () != editor)
    {
	property_window->remove ();
	property_window->add (*editor);
    }
    
    gchar *buffer = g_strdup_printf (_("Properties: %s"),
				     res->get_id ().c_str());
    property_window->set_title (buffer);
    g_free (buffer);
    
    property_window->show_all ();
    property_window->get_window ().raise ();
}

void FormWindow::toplevel_activate_cb ()
{
    show_form_property_editor ();
}

void FormWindow::widget_activate_cb (Widgets::Widget *widget)
{
    show_property_editor (widget);
}
